// njs_int.h

#ifndef INCLUDED_NJS_INT
#define INCLUDED_NJS_INT

#define NJS_DEBUG
#include <njs_system.h>
#include <njs_io.h>

namespace Njs {

   namespace Int {

      template <class Integer>
      Integer minimum (
      Integer i,                 // number being divided
      Integer j)                 // divisor 
      {
      return i < j ? i : j;
      }

      template <class Integer>
      Integer maximum (
      Integer i,                 // number being divided
      Integer j)                 // divisor 
      {
      return i > j ? i : j;
      }

      template <class Integer>
      Integer mod (              // Portable modular function %
      Integer i,                 // number being divided
      Integer j)                 // divisor 
      {
         Integer abs_j = j >= 0 ? j : -j;
         if (j == 0) {
      Njs::Io::abort ("Nks_Mod : j == 0");   
         }
         if (i < 0) {
            Integer k = (i >= 0 ? i : -i) % abs_j;
      return k == 0 ? k : abs_j - k;
         } else {
      return i % abs_j;
         }
      }

      template <class Integer>
      Integer euclidAlgorithm ( // Euclid's algorithm returning the greatest common divisor (i, j)
      Integer i_,
      Integer j_)
      {
         Integer abs_i = maximum <Integer> (i_ >= 0 ? i_ : -i_, j_ >= 0 ? j_ : -j_);
         Integer abs_j = minimum <Integer> (i_ >= 0 ? i_ : -i_, j_ >= 0 ? j_ : -j_);
         Integer remainder = 0;
         while (0 < abs_j) {
            remainder = mod <Integer> (abs_i, abs_j);
            abs_i = abs_j;
            abs_j = remainder;
         }
         return abs_i;
      }

      template <class Integer, class ConstIntegerPtr>
      Integer euclidAlgorithmVector ( // Euclid's algorithm returning the greatest common divisor (i, j)
      ConstIntegerPtr begin_,
      ConstIntegerPtr end_)
      {
         assert (begin_ <= end_);
         if (begin_ == end_) return 0;
         Integer gcd = *begin_;
         for (ConstIntegerPtr k = begin_ + 1; k != end_; k++) {
            gcd = euclidAlgorithm (gcd, *k);
         }
         return gcd;
      }

      template <class Integer>
      Integer minusOnePower (    // (-1)^i
      Integer i)                 // exponent
      {
         return mod <Integer> (i, 2) == 0 ? 1 : -1;
      }

      template <class Real, class Integer>
      Real integerPower (Real x, Integer n) // integer power function
      {
         if (x == 0.0) {
            if (n < 0) {
      Njs::Io::abortFormat("Function::integerPower <class Real, class Integer> : negative exponent of zero");
            } else if (n == 0) {
      return 1.0;
            } else {
      return 0.0;
            }
         }
         Real y = 1.0;
         Integer i = 0;
         for (i = n > 0 ? n : -n; i > 0 ; i /= 2) {
            if (i % 2 != 0) {
               y *= x;
            }
            x *= x;
         }
         if (n < 0) {
            y = 1.0 / y;
         }
      return y;
      }

      template <class Real, class UnsignedInteger>
      Real integerPositivePower (Real x, UnsignedInteger n) // integer power function
      {
         if (x == 0.0) {
            if (n == 0) {
      return 1.0;
            } else {
      return 0.0;
            }
         }
         Real y = 1.0;
         UnsignedInteger i = 0;
         for (i = n; i > 0 ; i /= 2) {
            if (i % 2 != 0) {
               y *= x;
            }
            x *= x;
         }
      return y;
      }

      template <class Int, class Integer>
      Int intPower (Int x, Integer n) // integer power function
      {
         assert (n >= 0);
         if (x == 0) {
      return n == 0 ? 1 : 0;
         }
         Int y = 1;
         Integer i = 0;
         for (i = n > 0 ? n : -n; i > 0 ; i /= 2) {
            if (i % 2 != 0) {
               y *= x;
            }
            x *= x;
         }
      return y;
      }

   }

}

#endif //! INCLUDED_NJS_INT




