// njs_io.cpp

#include <njs_io.h>

#undef NJS_DEBUG
#include <njs_debug.h>

#include <njs_ioutil.h>

using namespace Njs;

namespace {

   const size_t BUF_SIZE = 1024;

}

int Io::outStream ( // appends file
std::ostream &out_, // output stream
const char *format, // format string
...)
{
   char buf [BUF_SIZE];
	va_list args;
	va_start (args, format);
	int status = vsprintf (buf, format, args); // error = EOF
	va_end (args);
   out_ << buf;
return status;
}

int Io::outStream ( // appends file
std::ostream *out_, // output stream
const char *format, // format string
...)
{
   if (! out_) return 0;
   char buf [BUF_SIZE];
	va_list args;
	va_start (args, format);
	int status = vsprintf (buf, format, args); // error = EOF
	va_end (args);
   *out_ << buf;
return status;
}

int Io::appendAndCloseFile ( // opens, appends, and closes file
const char *nameOut, // output file : status : "a"
const char *format, // format string
...)
{
   if (nameOut == 0 || ! strcmp (nameOut, "")) {
return IoUtil::NULL_FILE;
   }
   std::ofstream out (nameOut, std::ios_base::app);
	if (! nameOut) {
return EOF;
	}
   char buf [BUF_SIZE];
	va_list args;
	va_start (args, format);
	int status = vsprintf (buf, format, args); // error = EOF
	va_end (args);
   out << buf;
return status;
}

int Io::writeAndCloseFile ( // opens, overwrites, and closes file
const char *nameOut, // output file : status : "a"
const char *format, // format string
...)
{
   if (nameOut == 0 || ! strcmp (nameOut, "")) {
return IoUtil::NULL_FILE;
   }
	va_list args;
	FILE *file = fopen (nameOut, "w");
	if (file == NULL) {
return EOF;
	}
	va_start (args, format);
	int status = vfprintf (file, format, args); // error = EOF
	va_end (args);
	fclose (file);
return status;
}

void Io::abortFormat ( // aborts with a formatted message
const char *format, // format string
...)
{
	va_list args;
	va_start (args, format);
	int status = vfprintf (stderr, format, args); // error = EOF
	va_end (args);
   (void) fprintf (stderr, "\n");
exit (1);
}

void Io::abort ( // aborts with a string
const char *message) // error message
{
   abortFormat ("%s", message);
}

char *Io::changePostfix ( // changes the postfix of a string
char *out_, // output name
const char *dat_, // input name
const char *outPostFix_, // output post-fix (0 = "out")
const char *datPostFix_, // input post-fix (0 = "dat")
char separator_) // separator ('\0' = '.')
{ // assumes sufficient storage in out
// adjust defaults
   const size_t DIM_BUFFER = 1024;
   char outPostFix [DIM_BUFFER];
   char datPostFix [DIM_BUFFER];
   strcpy (outPostFix, outPostFix_ ? outPostFix_ : "out"); 
   strcpy (datPostFix, datPostFix_ ? datPostFix_ : "dat"); 
   if (separator_ == '\0') separator_ = '.';
   strcpy (out_, dat_);
   char *sep = strrchr (out_, separator_);
   if (! sep || // the separator does not occur in out_
      (sep [1] != '\0' && strcmp (sep + 1, datPostFix) != 0)) { // datPostFix does not occur after separator
      sep = out_ + strlen (out_);
   }
   *sep = separator_;
   *++sep = '\0';
   strcat (out_, outPostFix_);
return out_;
}

bool Io::readLineToT ( // assumes buf can handle one line of input
std::istream &in, // input filestream
size_t dimBuf, // #(buf)
char *buf, // string before '!' from next line not filled with white space
const char *t) // termination characters (not white space)
{
   assert (t);
   size_t i = 0;
   size_t j = 0;
   size_t count = 0; 
   char *pbuf = buf;
   const char *pt = t;
   if (! in) Io::abort ("readLineThroughT : Input filestream is not valid.");
   while (in.getline (buf, dimBuf, '\n')) {
      if (dimBuf <= (count = (size_t) in.gcount ())) {
Io::abort ("readLineThroughT : Line is longer than buffer size.");
      }
      buf [count - 1] = '\0';
      for (pbuf = buf; *pbuf != '\0' && isspace (*pbuf); pbuf++); // advance past white space
      assert (*pbuf == '\0' || ! isspace (*pbuf));
      if (*pbuf == '\0') { // empty line
   continue;
      } else if (t [0] == '\0') { // no terminator
return true;
      } else if (strchr (t, *pbuf)) { // space followed immediately by terminator
   continue;
      } else for ( ; *pbuf != '\0' && ! strchr (t, *pbuf); pbuf++); // advance to terminator
      *pbuf = '\0'; // make the terminator the end of the string
return true;
   }
   buf [0] = '\0';
return false; // buf is empty and eof is reached 
}

void Io::stringToDouble ( // reads single real values from a string
char *str, // input : string
double *x) // output : real value 
{
	if (strcmp (str, "NaN") == 0) {
      *x = -HUGE_VAL;
   } else if (sscanf (str, "%lg", x) != 1) {
abortFormat ("stringToReal : %s does not contain a real number.", str);
   }
}

void Io::stringToLong ( // reads single real values from a string
char *str, // input : string
long *x) // output : long value 
{
	if (strcmp (str, "NaN") == 0) {
      *x = LONG_MAX;
   } else if (sscanf (str, "%ld", x) != 1) {
abortFormat ("stringToLong : %s does not contain a long number.", str);
   }
}


